'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _utilities = require('./../utilities');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parseOptions = function parseOptions(context) {
    return {
        always: (context.options[0] || 'always') === 'always'
    };
};

var propertyEvaluator = function propertyEvaluator(context, typeForMessage) {
    var _parseOptions = parseOptions(context);

    var always = _parseOptions.always;


    var sourceCode = context.getSourceCode();

    var getSpacesAfterColon = function getSpacesAfterColon(node, typeAnnotation) {
        if (node.type === 'FunctionTypeParam') {
            var colon = sourceCode.getTokenBefore(typeAnnotation);

            return typeAnnotation.start - colon.end;
        } else {
            var token = sourceCode.getFirstToken(typeAnnotation, 1);

            return token.start - typeAnnotation.start - 1;
        }
    };

    return function (node) {
        var parameterName = (0, _utilities.getParameterName)(node, context);
        var typeAnnotation = _lodash2.default.get(node, 'typeAnnotation') || _lodash2.default.get(node, 'left.typeAnnotation');

        if (typeAnnotation) {
            var spaceAfter = getSpacesAfterColon(node, typeAnnotation);

            if (always && spaceAfter > 1) {
                context.report(node, 'There must be 1 space after "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            } else if (always && spaceAfter === 0) {
                context.report(node, 'There must be a space after "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            } else if (!always && spaceAfter > 0) {
                context.report(node, 'There must be no space after "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            }
        }
    };
};

var returnTypeEvaluator = function returnTypeEvaluator(context) {
    var _parseOptions2 = parseOptions(context);

    var always = _parseOptions2.always;


    var sourceCode = context.getSourceCode();

    return function (functionNode) {
        // skip FunctionTypeAnnotation, possibly another rule as it's an arrow, not a colon?
        // (foo: number) => string
        //              ^^^^
        if (functionNode.returnType && functionNode.type !== 'FunctionTypeAnnotation') {
            var token = sourceCode.getFirstToken(functionNode.returnType, 1);
            var spaceAfter = token.start - functionNode.returnType.start - 1;

            if (always && spaceAfter > 1) {
                context.report(functionNode, 'There must be 1 space after return type colon.');
            } else if (always && spaceAfter === 0) {
                context.report(functionNode, 'There must be a space after return type colon.');
            } else if (!always && spaceAfter > 0) {
                context.report(functionNode, 'There must be no space after return type colon.');
            }
        }
    };
};

var functionEvaluators = (0, _utilities.iterateFunctionNodes)(function (context) {
    var checkParam = propertyEvaluator(context, 'parameter');
    var checkReturnType = returnTypeEvaluator(context);

    return function (functionNode) {
        _lodash2.default.forEach(functionNode.params, checkParam);
        checkReturnType(functionNode);
    };
});

var objectTypePropertyEvaluator = function objectTypePropertyEvaluator(context) {
    var _parseOptions3 = parseOptions(context);

    var always = _parseOptions3.always;


    var sourceCode = context.getSourceCode();

    return function (objectTypeProperty) {
        var colon = sourceCode.getFirstToken(objectTypeProperty, objectTypeProperty.optional ? 2 : 1);
        var typeAnnotation = objectTypeProperty.value;
        var name = (0, _utilities.getParameterName)(objectTypeProperty, context);

        var spaceAfter = typeAnnotation.start - colon.end;

        if (always && spaceAfter > 1) {
            context.report(objectTypeProperty, 'There must be 1 space after "' + name + '" type annotation colon.');
        } else if (always && spaceAfter === 0) {
            context.report(objectTypeProperty, 'There must be a space after "' + name + '" type annotation colon.');
        } else if (!always && spaceAfter > 0) {
            context.report(objectTypeProperty, 'There must be no space after "' + name + '" type annotation colon.');
        }
    };
};

exports.default = function (context) {
    return _extends({}, functionEvaluators(context), {
        ClassProperty: propertyEvaluator(context, 'class property'),
        ObjectTypeProperty: objectTypePropertyEvaluator(context)
    });
};

module.exports = exports['default'];