'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _utilities = require('./../utilities');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parseOptions = function parseOptions(context) {
    return {
        always: context.options[0] === 'always'
    };
};

var propertyEvaluator = function propertyEvaluator(context, typeForMessage) {
    var _parseOptions = parseOptions(context);

    var always = _parseOptions.always;


    var sourceCode = context.getSourceCode();

    var getSpacesBeforeColon = function getSpacesBeforeColon(node, typeAnnotation) {
        if (node.type === 'FunctionTypeParam') {
            // the colon isn't included in the typeAnnotation node here...
            var colon = sourceCode.getTokenBefore(typeAnnotation);
            var tokenBeforeColon = sourceCode.getTokenBefore(colon);

            return colon.start - tokenBeforeColon.end;
        } else {
            // tokenBeforeColon can be the identifier or the closing } token of a destructuring
            var _tokenBeforeColon = sourceCode.getTokenBefore(typeAnnotation, node.optional ? 1 : 0);

            return typeAnnotation.start - _tokenBeforeColon.end - (node.optional ? 1 : 0);
        }
    };

    return function (node) {
        var parameterName = (0, _utilities.getParameterName)(node, context);
        var typeAnnotation = _lodash2.default.get(node, 'typeAnnotation') || _lodash2.default.get(node, 'left.typeAnnotation');

        if (typeAnnotation) {
            var spaceBefore = getSpacesBeforeColon(node, typeAnnotation);

            if (always && spaceBefore > 1) {
                context.report(node, 'There must be 1 space before "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            } else if (always && spaceBefore === 0) {
                context.report(node, 'There must be a space before "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            } else if (!always && spaceBefore > 0) {
                context.report(node, 'There must be no space before "' + parameterName + '" ' + typeForMessage + ' type annotation colon.');
            }
        }
    };
};

var functionEvaluators = (0, _utilities.iterateFunctionNodes)(function (context) {
    var checkParam = propertyEvaluator(context, 'parameter');

    return function (functionNode) {
        _lodash2.default.forEach(functionNode.params, checkParam);
    };
});

var objectTypePropertyEvaluator = function objectTypePropertyEvaluator(context) {
    var _parseOptions2 = parseOptions(context);

    var always = _parseOptions2.always;


    var sourceCode = context.getSourceCode();

    return function (objectTypeProperty) {
        // tokenBeforeColon can be identifier, or a ? token if is optional
        var tokenBeforeColon = sourceCode.getFirstToken(objectTypeProperty, objectTypeProperty.optional ? 1 : 0);
        var colon = sourceCode.getFirstToken(objectTypeProperty, objectTypeProperty.optional ? 2 : 1);
        var name = (0, _utilities.getParameterName)(objectTypeProperty, context);

        var spaceBefore = colon.start - tokenBeforeColon.end;

        if (always && spaceBefore > 1) {
            context.report(objectTypeProperty, 'There must be 1 space before "' + name + '" type annotation colon.');
        } else if (always && spaceBefore === 0) {
            context.report(objectTypeProperty, 'There must be a space before "' + name + '" type annotation colon.');
        } else if (!always && spaceBefore > 0) {
            context.report(objectTypeProperty, 'There must be no space before "' + name + '" type annotation colon.');
        }
    };
};

exports.default = function (context) {
    return _extends({}, functionEvaluators(context), {
        ClassProperty: propertyEvaluator(context, 'class property'),
        ObjectTypeProperty: objectTypePropertyEvaluator(context)
    });
};

module.exports = exports['default'];